/***************************************************************************
                          HipReductionBuffer.h  -  description
                             -------------------
    begin                : June 18, 2020
    copyright            : (C) 2020 by Tomas Oberhuber
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

// Implemented by: Jakub Klinkovsky

#pragma once

#include <stdlib.h>

#include <TNL/Hip/CheckDevice.h>
#include <TNL/Exceptions/HipBadAlloc.h>
#include <TNL/Exceptions/HipSupportMissing.h>

namespace TNL {
namespace Algorithms {

class HipReductionBuffer
{
   public:
      inline static HipReductionBuffer& getInstance()
      {
         static HipReductionBuffer instance;
         return instance;
      }

      inline void setSize( size_t size )
      {
#ifdef HAVE_HIP
         if( size > this->size )
         {
            this->free();
            if( hipMalloc( ( void** ) &this->data, size ) != hipSuccess ) {
               this->data = 0;
               throw Exceptions::HipBadAlloc();
            }
            this->size = size;
         }
#else
         throw Exceptions::HipSupportMissing();
#endif
      }

      template< typename Type >
      Type* getData()
      {
         return ( Type* ) this->data;
      }

   private:
      // stop the compiler generating methods of copy the object
      HipReductionBuffer( HipReductionBuffer const& copy );            // Not Implemented
      HipReductionBuffer& operator=( HipReductionBuffer const& copy ); // Not Implemented

      // private constructor of the singleton
      inline HipReductionBuffer( size_t size = 0 )
      {
#ifdef HAVE_HIP
         setSize( size );
         atexit( HipReductionBuffer::free_atexit );
#endif
      }

      inline static void free_atexit( void )
      {
         HipReductionBuffer::getInstance().free();
      }

   protected:
      inline void free( void )
      {
#ifdef HAVE_HIP
         if( data )
         {
            hipFree( data );
            data = nullptr;
            TNL_CHECK_HIP_DEVICE;
         }
#endif
      }

      void* data = nullptr;

      size_t size = 0;
};

} // namespace Algorithms
} // namespace TNL
