/***************************************************************************
                          DeviceCallable.h  -  description
                             -------------------
    begin                : Jun 17, 2020
    copyright            : (C) 2020 by Tomas Oberhuber et al.
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

#pragma once

#ifdef HAVE_HIP
#include <hip/hip_runtime.h>
#endif

// The __device_callable__ macro has to be in a separate header file to avoid
// infinite loops by the #include directives.

/***
 * This macro serves for definition of function which are supposed to be called
 * even from device. If HAVE_CUDA or HAVE_HIP is defined, the __device_callable__ function
 * is compiled for both CPU and GPU. If neither HAVE_CUDA nor HAVE_HIP is defined, this macro has
 * no effect.
 */
#if defined HAVE_CUDA || defined HAVE_HIP
   #define __device_callable__ __host__ __device__
   #define __cuda_callable__   __host__ __device__
#else
   #define __device_callable__
   #define __cuda_callable__
#endif
