/***************************************************************************
                          Hip.h  -  description
                             -------------------
    begin                : Jun 17, 2020
    copyright            : (C) 2020 by Tomas Oberhuber
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

#pragma once

#ifdef HAVE_HIP
#include <hip/hip_runtime.h>
#endif

#include <TNL/String.h>
#include <TNL/Config/ConfigDescription.h>
#include <TNL/Config/ParameterContainer.h>

namespace TNL {
namespace Devices {

class Hip
{
public:
   static inline void configSetup( Config::ConfigDescription& config, const String& prefix = "" )
   {
#ifdef HAVE_HIP
      config.addEntry< int >( prefix + "hip-device", "Choose HIP device to run the computation.", 0 );
#else
      config.addEntry< int >( prefix + "hip-device", "Choose HIP device to run the computation (not supported on this system).", 0 );
#endif
   }

   static inline bool setup( const Config::ParameterContainer& parameters,
                             const String& prefix = "" )
   {
#ifdef HAVE_HIP
      int hipDevice = parameters.getParameter< int >( prefix + "hip-device" );
      if( hipSetDevice( hipDevice ) != hipSuccess )
      {
         std::cerr << "I cannot activate HIP device number " << hipDevice << "." << std::endl;
         return false;
      }
#endif
      return true;
   }
};

} // namespace Devices
} // namespace TNL
