/***************************************************************************
                          HipBadAlloc.h  -  description
                             -------------------
    begin                : Jun 17, 2020
    copyright            : (C) 2020 by Tomas Oberhuber et al.
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

// Implemented by: Jakub Klinkovsky

#pragma once

#include <new>

#ifdef HAVE_HIP
#include <hip/hip_runtime.h>
#endif

namespace TNL {
namespace Exceptions {

struct HipBadAlloc
   : public std::bad_alloc
{
   HipBadAlloc()
   {
#ifdef HAVE_HIP
      // Make sure to clear the CUDA error, otherwise the exception handler
      // might throw another exception with the same error.
      hipGetLastError();
#endif
   }

   const char* what() const throw()
   {
      return "Failed to allocate memory on the HIP device: "
             "most likely there is not enough space on the device memory.";
   }
};

} // namespace Exceptions
} // namespace TNL
