/***************************************************************************
                          HipRuntimeError.h  -  description
                             -------------------
    begin                : Jun 17, 2020
    copyright            : (C) 2020 by Tomas Oberhuber et al.
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

// Implemented by: Jakub Klinkovsky

#pragma once

#include "HipSupportMissing.h"

namespace TNL {
namespace Exceptions {

#ifdef HAVE_HIP
   using HipStatusType = hipError_t;
#else
   using HipStatusType = int;
#endif

class HipRuntimeError
   : public std::runtime_error
{
public:
   HipRuntimeError( HipStatusType error_code )
   : std::runtime_error( "HIP ERROR " + std::to_string( (int) error_code ) + " (" + name( error_code ) + "): "
                         + description( error_code ) + "." ),
     code_( error_code )
   {}

   HipRuntimeError( HipStatusType error_code, const std::string& what_arg )
   : std::runtime_error( "HIP ERROR " + std::to_string( (int) error_code ) + " (" + name( error_code ) + "): "
                         + description( error_code ) + ".\nDetails: " + what_arg ),
     code_(error_code)
   {}

   HipRuntimeError( HipStatusType error_code, const char* file_name, int line )
   : std::runtime_error( "HIP ERROR " + std::to_string( (int) error_code ) + " (" + name( error_code ) + "): "
                         + description( error_code ) + ".\nSource: line " + std::to_string( line )
                         + " in " + file_name + ": " + description( error_code ) ),
     code_(error_code)
   {}

   HipStatusType code() const
   {
      return code_;
   }

private:
   static std::string name( HipStatusType error_code )
   {
#ifdef HAVE_HIP
      return hipGetErrorName( error_code );
#else
      throw HipSupportMissing();
#endif
   }

   static std::string description( HipStatusType error_code )
   {
#ifdef HAVE_HIP
      return hipGetErrorString( error_code );
#else
      throw HipSupportMissing();
#endif
   }

   HipStatusType code_;
};

} // namespace Exceptions
} // namespace TNL
