/***************************************************************************
                          CheckDevice.h  -  description
                             -------------------
    begin                : Jun 17, 2020
    copyright            : (C) 2020 by Tomas Oberhuber et al.
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

#pragma once

#include <TNL/Exceptions/HipRuntimeError.h>

namespace TNL {
namespace Hip {

#ifdef HAVE_HIP
   /****
    * I do not know why, but it is more reliable to pass the error code instead
    * of calling hipGetLastError() inside the function.
    * We recommend to use macro 'TNL_CHECK_HIP_DEVICE' defined bellow.
    */
   inline void checkDevice( const char* file_name, int line, hipError_t error )
   {
      if( error != hipSuccess )
         throw Exceptions::HipRuntimeError( error, file_name, line );
   }
#else
   inline void checkDevice() {}
#endif

} // namespace Hip
} // namespace TNL

#ifdef HAVE_HIP
#define TNL_CHECK_HIP_DEVICE ::TNL::Hip::checkDevice( __FILE__, __LINE__, hipGetLastError() )
#else
#define TNL_CHECK_HIP_DEVICE ::TNL::Hip::checkDevice()
#endif
