/***************************************************************************
                          MemoryHelpers.h  -  description
                             -------------------
    begin                : Jun 18, 2020
    copyright            : (C) 2020 by Tomas Oberhuber et al.
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

#pragma once

#include <iostream>

#include <TNL/Hip/CheckDevice.h>
#include <TNL/Exceptions/HipSupportMissing.h>
#include <TNL/Exceptions/HipBadAlloc.h>

namespace TNL {
namespace Hip {

template< typename ObjectType >
[[deprecated("Allocators and MemoryOperations should be used instead.")]]
ObjectType* passToDevice( const ObjectType& object )
{
#ifdef HAVE_Hip
   ObjectType* deviceObject;
   if( hipMalloc( ( void** ) &deviceObject,
                   ( size_t ) sizeof( ObjectType ) ) != hipSuccess )
      throw Exceptions::HipBadAlloc();
   if( hipMemcpy( ( void* ) deviceObject,
                   ( void* ) &object,
                   sizeof( ObjectType ),
                   hipMemcpyHostToDevice ) != hipSuccess )
   {
      TNL_CHECK_HIP_DEVICE;
      hipFree( ( void* ) deviceObject );
      TNL_CHECK_HIP_DEVICE;
      return 0;
   }
   return deviceObject;
#else
   throw Exceptions::HipSupportMissing();
#endif
}

template< typename ObjectType >
[[deprecated("Allocators and MemoryOperations should be used instead.")]]
ObjectType passFromDevice( const ObjectType* object )
{
#ifdef HAVE_HIP
   ObjectType aux;
   hipMemcpy( ( void* ) aux,
               ( void* ) &object,
               sizeof( ObjectType ),
               hipMemcpyDeviceToHost );
   TNL_CHECK_HIP_DEVICE;
   return aux;
#else
   throw Exceptions::HipSupportMissing();
#endif
}

template< typename ObjectType >
[[deprecated("Allocators and MemoryOperations should be used instead.")]]
void passFromDevice( const ObjectType* deviceObject,
                     ObjectType& hostObject )
{
#ifdef HAVE_HIP
   hipMemcpy( ( void* ) &hostObject,
               ( void* ) deviceObject,
               sizeof( ObjectType ),
               hipMemcpyDeviceToHost );
   TNL_CHECK_HIP_DEVICE;
#else
   throw Exceptions::HipSupportMissing();
#endif
}

template< typename ObjectType >
[[deprecated("Allocators and MemoryOperations should be used instead.")]]
void freeFromDevice( ObjectType* deviceObject )
{
#ifdef HAVE_HIP
   hipFree( ( void* ) deviceObject );
   TNL_CHECK_HIP_DEVICE;
#else
   throw Exceptions::HipSupportMissing();
#endif
}

template< typename ObjectType >
void print( const ObjectType* deviceObject, std::ostream& str = std::cout )
{
#ifdef HAVE_HIP
   ObjectType hostObject;
   passFromDevice( deviceObject, hostObject );
   str << hostObject;
#endif
}

} // namespace Hip
} // namespace TNL
